/**
 * @(#)RW_DataModel.java  12/12/99
 * @author Peter Au
 * All rights reserved.
 */

package vishnu.testvis.treemap.table;

import vishnu.testvis.object.ClusterObj;
import vishnu.testvis.object.DocObj;
import javax.swing.table.DefaultTableModel;
import java.util.*;
import vishnu.testvis.sammon.QSortAlgorithmTableLength;
import vishnu.testvis.object.LengthTable;

/**
 * This table contains 3 columns:
 *   - a tick box
 *   - a related word
 *   - and the hit document frequency
 */
public class RW_DataModel extends DefaultTableModel{
static final private String[] __COLNAMES={"","Related Word","Hit Doc Freq"};
private String[] _related_words;
private ClusterObj _cObj;
private Vector _colnames = new Vector(__COLNAMES.length);

private Vector _docs = null;  // this is created inside 'setData'


  public RW_DataModel()
  {
    //-- setup the column's names
    for (int i=0;i<__COLNAMES.length;i++)
      _colnames.addElement(__COLNAMES[i]);
    clear();
  }


  //-- clear all data variables in this data model
  public void clear()
  {
    _related_words = null;
    _cObj = null;
    _docs = null;
    setDataVector(new Vector(0), _colnames);
  }


  //--------------------------------------------------------------------------
  /**
   * Set up the cluster data in the data model
   * pre: the related-words are new to the data model
   * @param rw is a list of related words
   * @param c is the data of the cluster
   */
  public void setData(String[] rw, ClusterObj c)
  {
    Vector dataVec = new Vector(0);
    Vector hdfVec;
    Vector docs = new Vector();
    _related_words = rw;
    _cObj = c;

    hdfVec = _cObj.getHitDocFreq();
    _docs = new Vector();
// invarient: _hdfVec.size() == rw.length

    //-- for each related word do
    for(int i=0;i<hdfVec.size();i++){
      int freq = ((Vector)hdfVec.elementAt(i)).size();
      if (freq>0)
      {
        Vector  entry;
        entry = new Vector(__COLNAMES.length+1);
        entry.addElement(new Boolean(false));
        entry.addElement(_related_words[i]);
        entry.addElement(new Integer(freq*100/_cObj.getDocsSize()));
        dataVec.addElement(entry);

        /**
         * This extra cell is for internal used only.
         * each entry consists of a vector of doc, those documents contain the
         * related-word (with weight>0)
         *
         * This is used in the filtering routine
         * so if I sort the above I must likewise sort this one.
         */
         docs.addElement(hdfVec.elementAt(i));
      }
    }  // for
    LengthTable [] table = new LengthTable[dataVec.size()];
    for (int i=0;i<dataVec.size();i++)
    {
      Vector ent =  (Vector)dataVec.elementAt(i);
      Integer in = (Integer)ent.elementAt(2);
      table[i]=new LengthTable(i,in.intValue());
    }
    QSortAlgorithmTableLength qSort = new QSortAlgorithmTableLength();
    try
    {
      qSort.sort(table);
    }
    catch (Exception e)
    {}

    Vector newDataVec= new Vector(dataVec.size());
    for (int i=dataVec.size()-1;i>=0;i--)
    {
      int index = table[i].index;
      newDataVec.addElement(dataVec.elementAt(index));
      _docs.addElement(docs.elementAt(index));

    }

    setDataVector(newDataVec, _colnames);
  } // setData
  
  // overloaded, this one for dendro
  // we pass on freq vector instead of obj
  public void setData(String[] rw, vishnu.testvis.dendro.ClusterObj _cObj)
  {
    Vector dataVec = new Vector(0);
    Vector hdfVec;
    Vector docs = new Vector();
    _related_words = rw;

    hdfVec = _cObj.getHitDocFreq();
    _docs = new Vector();
// invarient: _hdfVec.size() == rw.length

    //-- for each related word do
    for(int i=0;i<hdfVec.size();i++){
      int freq = ((Vector)hdfVec.elementAt(i)).size();
      if (freq>0)
      {
        Vector  entry;
        entry = new Vector(__COLNAMES.length+1);
        entry.addElement(new Boolean(false));
        entry.addElement(_related_words[i]);
        entry.addElement(new Integer(freq*100/_cObj.getDocsSize()));
        dataVec.addElement(entry);

        /**
         * This extra cell is for internal used only.
         * each entry consists of a vector of doc, those documents contain the
         * related-word (with weight>0)
         *
         * This is used in the filtering routine
         * so if I sort the above I must likewise sort this one.
         */
         docs.addElement(hdfVec.elementAt(i));
      }
    }  // for
    LengthTable [] table = new LengthTable[dataVec.size()];
    for (int i=0;i<dataVec.size();i++)
    {
      Vector ent =  (Vector)dataVec.elementAt(i);
      Integer in = (Integer)ent.elementAt(2);
      table[i]=new LengthTable(i,in.intValue());
    }
    QSortAlgorithmTableLength qSort = new QSortAlgorithmTableLength();
    try
    {
      qSort.sort(table);
    }
    catch (Exception e)
    {}

    Vector newDataVec= new Vector(dataVec.size());
    for (int i=dataVec.size()-1;i>=0;i--)
    {
      int index = table[i].index;
      newDataVec.addElement(dataVec.elementAt(index));
      _docs.addElement(docs.elementAt(index));

    }

    setDataVector(newDataVec, _colnames);
  } // setData


  public String[] getRefinedTrecIDs()
  {
      Vector sVec = new Vector();
      HashSet set = new HashSet();  //-- this can avoid duplicated items !
      boolean isNewItem;
      
      for(int i=0;i<getDataVector().size();i++)
      {
            if (((Boolean) getValueAt(i,0)).booleanValue())
            {
                
                Vector docs = (Vector) _docs.elementAt(i);

                for (int j=0;j<docs.size();j++)
                {
                    isNewItem = set.add((DocObj) docs.elementAt(j));
                    if (isNewItem)
                    {
                        sVec.addElement(((DocObj)docs.elementAt(j)).getTrecID());
                        //System.out.println(((DocObj) docs.elementAt(j)).getTrecID());
                    }
                }
            }
      }
      
      String[] sStr = new String[sVec.size()];
      for( int i = 0; i < sVec.size(); i++ )
      {
          sStr[i] = new String((String)sVec.elementAt(i));
      }
      return sStr;
  }
  
  public String[] getFilterWords()
  {
        Vector sVect = new Vector();
        
        for(int i=0;i<getDataVector().size();i++)
        {
            if (((Boolean) getValueAt(i,0)).booleanValue())
            {
                sVect.addElement((String)getValueAt(i,1));
            }   
        }
        
        String sStr[] = new String[sVect.size()];
        
        for( int i = 0; i < sVect.size(); i++ )
        {
            sStr[i] = (String)sVect.elementAt(i);    
        }
        return sStr;
  }

  public String getRefinedDocs(){
  String s = "";
  HashSet set = new HashSet();  //-- this can avoid duplicated items !
  boolean isNewItem;
    for(int i=0;i<getDataVector().size();i++){

      if (((Boolean) getValueAt(i,0)).booleanValue()) {
       Vector docs = (Vector) _docs.elementAt(i);

        for (int j=0;j<docs.size();j++){
          isNewItem = set.add((DocObj) docs.elementAt(j));
          if (isNewItem)
          {
            s += ((DocObj) docs.elementAt(j)).format4Fields();
            //System.out.println(((DocObj) docs.elementAt(j)).format4Fields());
          }
            
        }

      }

    }
    s = "<html><font face=Arial size=+10> " + set.size()  + " documents found\n" + s +
        "</font></html>\n";
    return s;
  }




  //--------------------------------------------------------------------------
  public Class getColumnClass(int col)
  {
    switch (col)
    {
      case 0: return Boolean.class;
      case 1: return String.class;
      case 2: return Integer.class;
      default: return Object.class;
    }
  }

  public int getColumnCount(){return __COLNAMES.length;}

  public boolean isCellEditable(int row, int col){
    return ((col==0) ? true : false);
  }
  public void setValueAt(Object value, int row, int col){
    super.setValueAt(value,row,col);
  }


}
