package org.greenstone.gsdl3.gs3build.metadata;

import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import org.greenstone.gsdl3.gs3build.util.MultiMap;
import org.greenstone.gsdl3.gs3build.util.XMLTools;

/**
 *  Implement a simple, unordered, namespace in METS.
 *
 *  @see: org.greenstone.gsdl3.gs3build.metadata.METSNamespace
 */

public class GSDL3Namespace extends SimpleNamespace
{
	public static String GSDL3_NAMESPACE_ID = "gsdl3";
	private static String GSDL3_METADATA_TAG = "Metadata";

	public GSDL3Namespace()
	{ super(GSDL3_NAMESPACE_ID);
	}

	public GSDL3Namespace(METSLocation location)
	{ super(GSDL3_NAMESPACE_ID, location);
	}

	public GSDL3Namespace(Element mdWrapTag)
	{	super(GSDL3_NAMESPACE_ID);

		NodeList childNodes = mdWrapTag.getChildNodes();
		for (int c = 0; c < childNodes.getLength(); c ++)
		{	// a metadata node
			if (childNodes.item(c).getNodeType() == org.w3c.dom.Node.ELEMENT_NODE)
			{	String metadataLabel = childNodes.item(c).getNodeName();

				// cull any leading "GSDL3" tag and its colon...
				if (metadataLabel.startsWith(GSDL3_NAMESPACE_ID + ":"))
				{	metadataLabel = metadataLabel.substring(this.name.length() + 1);
				}

				// TODO: throw error: bad metadata type
				if (metadataLabel.equals(GSDL3_METADATA_TAG) == false)
				{	continue;
				}

				// Get the label - if the name isn't set, abort.
				// TODO: throw error: incomplete metadata tag
				metadataLabel = ((Element) childNodes.item(c)).getAttribute("name");
				if (metadataLabel == null)
				{ continue;
				}

				// now build the associated value.
				String metadataValue = "";

				Element childElement = (Element) childNodes.item(c);
				for (int i = 0; i < childElement.getChildNodes().getLength(); i ++)
				{	metadataValue = metadataValue + childElement.getChildNodes().item(i).toString();
				}

				this.addMetadata(metadataLabel, metadataValue);
			}
		}
	}

	/**
	 *  Write out a single element - this may be overloaded to provide for the
	 *  appropriate formatting for this metadata.
	 */
	protected boolean writeItem(PrintWriter writer, String label)
	{	if (this.location == null)
		{ // just place the metadata in a simple wrapper
			Iterator values = this.metadataMap.getAll(label).iterator();

			while (values.hasNext())
			{	String tag;
				String value = values.next().toString();

				tag = XMLTools.getOpenTag(GSDL3_NAMESPACE_ID, GSDL3_METADATA_TAG);
				tag = XMLTools.addAttribute(tag, "name", label);
				writer.print(tag);
				writer.print(value);

				writer.println(XMLTools.getCloseTag(this.name, GSDL3_METADATA_TAG));
			}
		}
		else
		{	String tag = XMLTools.getOpenTag("mets", "mdRef");
			tag = XMLTools.addAttribute(tag, "LOCTYPE", this.location.getType());
			tag = XMLTools.addAttribute(tag, "xlink:href", this.location.getLocation().toString());
			tag = XMLTools.addAttribute(tag, "MDTYPE", this.getName());
			tag = XMLTools.makeSingleton(tag);
			writer.println(tag);
		}
		return true;
	}
}