package org.greenstone.gsdl3.util;

import org.greenstone.gsdl3.gs3build.util.GS3SQLConnectionFactory;
import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;

import java.sql.ResultSet;
import java.sql.SQLException;

import java.util.ArrayList;

public class SQLQuery {
    GS3SQLConnection connection = null;
    
    public SQLQuery() {

    }

    public boolean setDatabase(String db_name) {
	try {
	    connection = GS3SQLConnectionFactory.createConnection(db_name);
	    
	    // test the connection
	    String query = "select * from "+GSSQL.DOCUMENT_TABLE+GSSQL.END;
	    if (!connection.execute(query)) {
		connection = null;
		return false;
	    }
	    return true;
	} catch (Exception e) {
	    System.err.println("SQLQuery.setDatabase Exception");
	    return false;
	}
    }

    public String MGNum2OID(String mg_num) {
	System.out.println("converting mg num to oid "+mg_num);
	// get doc id and meta id
	String query = "select "+ 
	    GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_ID+
	    GSSQL.COMMA+GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ID+
	    " from "+GSSQL.METADATA_VALUE_TABLE+GSSQL.COMMA+GSSQL.NAMESPACE_TABLE+GSSQL.COMMA+GSSQL.METADATA_TABLE+
	    " where "+
	    GSSQL.METADATA_VALUE_TABLE+GSSQL.DOT+GSSQL.LABEL+GSSQL.EQUALS_QUOTE+GSSQL.MG_NUM+GSSQL.QUOTE+
	    " and "+GSSQL.METADATA_VALUE_TABLE+GSSQL.DOT+GSSQL.VALUE+GSSQL.EQUALS_QUOTE+mg_num+GSSQL.QUOTE+
	    " and "+GSSQL.METADATA_VALUE_TABLE+GSSQL.DOT+GSSQL.NAMESPACE_REF+GSSQL.EQUALS+GSSQL.NAMESPACE_TABLE+GSSQL.DOT+GSSQL.NAMESPACE_REF+
	    " and "+GSSQL.NAMESPACE_TABLE+GSSQL.DOT+GSSQL.METADATA_REF+GSSQL.EQUALS+GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_REF+
	    GSSQL.END;
	
	connection.execute(query);
	ResultSet results = null;
	String doc_id = null;
	String meta_id = null;
	try {
	    results = connection.getResultSet();
	    results.first();
	    doc_id = results.getString(GSSQL.DOCUMENT_ID);
	    meta_id = results.getString(GSSQL.METADATA_ID);
	} catch (java.sql.SQLException e) {

	    System.err.println("SQLQuery.MGNum2OID() Error: "+e.getMessage());
	    return null;
	}
	System.out.println("doc id ="+doc_id+", meta id = "+meta_id);

	// now get division label
	query = "select "+
	    GSSQL.DIVISION_TABLE+GSSQL.DOT+GSSQL.SHORT_LABEL+
	    " from "+GSSQL.DIVISION_TABLE+GSSQL.COMMA+GSSQL.DIVISION_METADATA_TABLE+
	    " where "+GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_ID+GSSQL.EQUALS_QUOTE+meta_id+GSSQL.QUOTE+
	    " and "+GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+
	    " and "+GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.DIVISION_REF+GSSQL.EQUALS+GSSQL.DIVISION_TABLE+GSSQL.DOT+GSSQL.DIVISION_REF+
	    GSSQL.END;

	connection.execute(query);
	String short_label = null;
	try {
	    results = connection.getResultSet();
	    results.first();
	    short_label = results.getString(GSSQL.SHORT_LABEL);

	} catch (java.sql.SQLException e) {

	    System.err.println("SQLQuery.MGNum2OID() Error: "+e.getMessage());
	    return null;
	}

	///ystem.out.println("short label = "+short_label);

	System.out.println("final doc_id = "+GS3OID.createOID(doc_id, short_label));
	return GS3OID.createOID(doc_id, short_label);
    }

    public String OID2MGNum(String oid) {
	System.out.println("converting oid "+oid+" to mg num");
	String id = getDocumentMetadata(oid, "gsdl3.mgseqno");
	System.out.println("mg id = "+id);
	return id;

    }

    // classifier metadata is not actually metadata
    public String getClassifierMetadata(String oid, String full_meta_name) {

	if (full_meta_name.equals("Title") || full_meta_name.equals("numleafdocs")) {
	    // get the description
	    String field_name="";
	    if (full_meta_name.equals("Title")) {
		field_name = GSSQL.DESCRIPTION;
	    } else {
		field_name = GSSQL.NUM_LEAF_DOCUMENTS;
	    }
	     
	    String query = "select "+ field_name+
		" from " + GSSQL.CLASSIFIER_TABLE +
		" where " + GSSQL.CLASSIFIER_ID + GSSQL.EQUALS_QUOTE + oid + GSSQL.QUOTE +
		GSSQL.END;
	    
	    connection.execute(query);
	    ResultSet results = null;
	    String value = null;
	    try {
		results = connection.getResultSet();
		results.first();
		value = results.getString(field_name);
	    } catch (java.sql.SQLException e) {
		
		System.err.println("SQLQuery.getClassifierMetadata() Error: "+e.getMessage());
		return null;
	    }
	    return value;
 
	}  else return null;
    
    }
    
    public ArrayList getDocumentChildren(String oid) {

	ArrayList children = new ArrayList();
	String doc_id = oid;
	String section_id = "All"; // if we are just passed a doc id
	if (!GS3OID.isDocTop(oid)) {
	    doc_id = GS3OID.getDocID(oid);
	    section_id = GS3OID.getSectionLabel(oid);
	}
	System.out.println("getDocChildren: oid = "+oid+", doc id = "+doc_id+", sec id ="+section_id);
//  	int sep_index = oid.indexOf(".");
//  	if (sep_index != -1) {
//  	    doc_id = oid.substring(0, sep_index);
//  	    section_id = oid.substring(sep_index+1);
//  	}

	// get the division ref for this section
	String query = "select "+GSSQL.DIVISION_REF +
	    " from "+GSSQL.DIVISION_TABLE+
	    " where "+GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+
	    " and "+GSSQL.SHORT_LABEL+GSSQL.EQUALS_QUOTE+section_id+GSSQL.QUOTE+
	    GSSQL.END;

	connection.execute(query);
	ResultSet results = null;
	String div_ref = null;
	try {
	    results = connection.getResultSet();
	    results.first();
	    div_ref = results.getString(GSSQL.DIVISION_REF);
	} catch (java.sql.SQLException e) {
	    
	    System.err.println("SQLQuery.getDocumentChildren() Error1: "+e.getMessage());
	    return null;
	}

	// now get the list of children
	query = "select "+GSSQL.SHORT_LABEL +
	    " from "+GSSQL.DIVISION_TABLE+
	    " where "+ GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+
	    " and "+GSSQL.PARENT_TYPE+GSSQL.EQUALS_QUOTE+GSSQL.DIVISION+GSSQL.QUOTE+
	    " and "+GSSQL.PARENT_REF+GSSQL.EQUALS_QUOTE+div_ref+GSSQL.QUOTE+
	    GSSQL.END;
	connection.execute(query);
	results = null;
	try {
	    results = connection.getResultSet();
	    while (results.next()) {
		String id = results.getString(GSSQL.SHORT_LABEL);
		children.add(GS3OID.createOID(doc_id, id));
	    }
	} catch (java.sql.SQLException e) {
	    
	    System.err.println("SQLQuery.getDocumentChildren() Error2: "+e.getMessage());
	    return null;
	}

	return children;
	
    }
    
    public boolean documentHasChildren(String oid) {
	// for now, use getChildren, but can we do this a better way??
	ArrayList children = getDocumentChildren(oid);
	if (children==null || children.size()==0) {
	    return false;
	}
	return true;
	
    }
    public ArrayList getClassifierChildren(String oid) {

	ArrayList children = new ArrayList();

	String query = "select "+ GSSQL.CLASSIFIER_ID +
	    " from " + GSSQL.CLASSIFIER_TABLE +
	    " where " + GSSQL.PARENT_ID + GSSQL.EQUALS_QUOTE + oid + GSSQL.QUOTE +
	    " and " + GSSQL.NUM_LEAF_DOCUMENTS + GSSQL.NOT+GSSQL.EQUALS_QUOTE+ "0"+GSSQL.QUOTE +
	    GSSQL.END; 
	
	connection.execute(query);
	ResultSet results = null;
	try {
	    results = connection.getResultSet();
	    while (results.next()) {
		String id = results.getString(GSSQL.CLASSIFIER_ID);
		children.add(id);
	    }
	} catch (java.sql.SQLException e) {
		
	    System.err.println("SQLQuery.getClassifierChildren Error: "+e.getMessage());
	    return null;
	}
	return children;
    }

    public ArrayList getClassifierDocChildren(String oid) {
	System.out.println("in getclassifierdocchildren");
	ArrayList children = new ArrayList();

	String query = "select "+ GSSQL.CLASS_DOCUMENT_TABLE+GSSQL.DOT+ GSSQL.DOCUMENT_ID+
	    " from " + GSSQL.CLASSIFIER_TABLE + GSSQL.COMMA+GSSQL.CLASS_DOCUMENT_TABLE+
	    " where " + GSSQL.CLASSIFIER_TABLE +GSSQL.DOT+ GSSQL.CLASSIFIER_ID + GSSQL.EQUALS_QUOTE + oid + GSSQL.QUOTE +
	    " and " + GSSQL.CLASSIFIER_TABLE +GSSQL.DOT+ GSSQL.CLASSIFIER_REF+GSSQL.EQUALS+GSSQL.CLASS_DOCUMENT_TABLE+GSSQL.DOT+ GSSQL.CLASSIFIER_REF+
	    " order by "+GSSQL.CLASS_DOCUMENT_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ORDER+
	    GSSQL.END;
	System.out.println("query="+query);
	connection.execute(query);
	ResultSet results = null;
	try {
	    results = connection.getResultSet();
	    while (results.next()) {
		String id = results.getString(GSSQL.DOCUMENT_ID);
		children.add(id);
	    }
	} catch (java.sql.SQLException e) {
		
	    System.err.println("SQLQuery.getClassifierDocChildren() Error: "+e.getMessage());
	    return null;
	}
	// now check for documents
	
	return children;
    }
    
    public String getDocumentMetadata(String oid, String full_meta_name) {

	String doc_id = oid;
	String section_id = "All"; // if we are just passed a doc id
	if (!GS3OID.isDocTop(oid)) {
	    doc_id = GS3OID.getDocID(oid);
	    section_id = GS3OID.getSectionLabel(oid);
	}
//  	int sep_index = oid.indexOf(".");
//  	if (sep_index != -1) {
//  	    doc_id = oid.substring(0, sep_index);
//  	    section_id = oid.substring(sep_index+1);
//  	}

	System.out.println("get meta for "+oid+", doc id = "+doc_id+"sec id = "+section_id);

	// get the metadata block id
	String query = "select "+ 
	    GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_ID +
	    " from " + GSSQL.DIVISION_METADATA_TABLE+GSSQL.COMMA+GSSQL.DIVISION_TABLE+
	    " where " + GSSQL.DIVISION_TABLE+GSSQL.DOT+GSSQL.DIVISION_REF+GSSQL.EQUALS+GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.DIVISION_REF+
	    " and " + GSSQL.DIVISION_TABLE+GSSQL.DOT+GSSQL.SHORT_LABEL+GSSQL.EQUALS_QUOTE+section_id+GSSQL.QUOTE+
	    " and " + GSSQL.DIVISION_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+
	    " and "+GSSQL.DIVISION_METADATA_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+ 
	    GSSQL.END;
	
	connection.execute(query);
	ResultSet results = null;
	String meta_id = null;
	try {
	    results = connection.getResultSet();
	    results.first();
	    meta_id = results.getString(GSSQL.METADATA_ID);
	} catch (java.sql.SQLException e) {

	    System.err.println("SQLQuery.getDocumentMetadata() Error: "+e.getMessage());
	    return null;
	}
	System.err.println("metadata block id = "+meta_id);

	// now get the list of namespace refs for the right namespace
	int sep_index = full_meta_name.indexOf('.');
	String meta_ns = "gsdl3";
	String meta_name = full_meta_name;
	if (sep_index != -1) {

	    meta_ns = full_meta_name.substring(0, sep_index);
	    meta_name = full_meta_name.substring(sep_index+1);
	}
	System.out.println("orig meta = "+full_meta_name+", ns = "+meta_ns+", name="+meta_name);

	query = "select * "+ 
	    " from "+GSSQL.NAMESPACE_TABLE+GSSQL.COMMA+GSSQL.METADATA_TABLE+GSSQL.COMMA+GSSQL.METADATA_VALUE_TABLE+
	    " where "+GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_ID+GSSQL.EQUALS_QUOTE+meta_id+GSSQL.QUOTE+
	    " and "+ GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.DOCUMENT_ID+GSSQL.EQUALS_QUOTE+doc_id+GSSQL.QUOTE+
	    " and " + GSSQL.METADATA_TABLE+GSSQL.DOT+GSSQL.METADATA_REF+GSSQL.EQUALS+GSSQL.NAMESPACE_TABLE+GSSQL.DOT+GSSQL.METADATA_REF+
	    " and "+ GSSQL.NAMESPACE_TABLE+GSSQL.DOT+GSSQL.NAMESPACE_TYPE+GSSQL.EQUALS_QUOTE+meta_ns+GSSQL.QUOTE+
	    " and "+GSSQL.NAMESPACE_TABLE+GSSQL.DOT+GSSQL.NAMESPACE_REF+GSSQL.EQUALS+GSSQL.METADATA_VALUE_TABLE+GSSQL.DOT+GSSQL.NAMESPACE_REF+
	    GSSQL.END;
	System.out.println("query="+query);
	connection.execute(query);
	String meta_value = null;
	try {
	    results = connection.getResultSet();
	    while (results.next()) {
		String m_name = results.getString(GSSQL.LABEL);
		if (meta_name.equals(m_name)) {
		    meta_value = results.getString(GSSQL.VALUE);
		    break;
		}
	    }
	} catch (java.sql.SQLException e) {

	    System.err.println("SQLQuery.getDocumentMetadata() Error: "+e.getMessage());
	    return null;
	}
	
	return meta_value;
    }
    public static void main (String [] args) {

	SQLQuery self = new SQLQuery();
	self.setDatabase("gs3test");
	System.out.println("getting oid for mg num 3");
	String oid = self.MGNum2OID("stx.3");
	System.out.println("oid was "+oid);
	
	System.out.println("title metadata for mg num 3:");
	String meta = self.getDocumentMetadata(oid, "gsdl3.title");
	if (meta != null) {System.out.println(meta);}
	System.out.println("subject metadata for mg num 3:");
	meta = self.getDocumentMetadata(oid, "gsdl3.Subject");
	if (meta != null) {System.out.println(meta);}

	System.out.println("converting back to mg, oid "+oid);
	String id = self.OID2MGNum(oid);
	System.out.println(""+id);

	System.out.println("getting oid for mg num 8");
	oid = self.MGNum2OID("stx.8");
	System.out.println("oid was "+oid);

	System.out.println("title metadata for mg num 8:");
	meta = self.getDocumentMetadata(oid, "gsdl3.title");
	if (meta != null) {System.out.println(meta);}
	System.out.println("subject metadata for mg num 8:");
	meta = self.getDocumentMetadata(oid, "gsdl3.Subject");
	if (meta != null) {System.out.println(meta);}

	System.out.println("converting back to mg, oid "+oid);
	id = self.OID2MGNum(oid);
	System.out.println(""+id);
    }




}
